@@ -459,6 +459,8 @@ raptor_new_parser(const char *name) {
 
   rdf_parser->failed=0;
 
+  rdf_parser->error_handlers.locator=&rdf_parser->locator;
+  
   /* Initialise default (lax) feature values */
   raptor_set_parser_strict(rdf_parser, 0);
 
@@ -557,6 +559,9 @@ int
 raptor_parse_chunk(raptor_parser* rdf_parser,
                    const unsigned char *buffer, size_t len, int is_end) 
 {
+  if(rdf_parser->sb)
+    raptor_stringbuffer_append_counted_string(rdf_parser->sb, buffer, len, 1);
+    
   return rdf_parser->factory->chunk(rdf_parser, buffer, len, is_end);
 }
 
@@ -583,6 +588,9 @@ raptor_free_parser(raptor_parser* rdf_parser)
   if(rdf_parser->default_generate_id_handler_prefix)
     RAPTOR_FREE(cstring, rdf_parser->default_generate_id_handler_prefix);
 
+  if(rdf_parser->sb)
+    raptor_free_stringbuffer(rdf_parser->sb);
+
   RAPTOR_FREE(raptor_parser, rdf_parser);
 }
 
@@ -811,8 +819,8 @@ raptor_parse_uri_with_connection(raptor_parser* rdf_parser, raptor_uri *uri,
   else if(rdf_parser->features[RAPTOR_FEATURE_NO_NET])
     raptor_www_set_uri_filter(www, raptor_parse_uri_no_net_filter, rdf_parser);
   
-  raptor_www_set_error_handler(www, rdf_parser->error_handler, 
-                               rdf_parser->error_user_data);
+  raptor_www_set_error_handler(www, rdf_parser->error_handlers.error_handler, 
+                               rdf_parser->error_handlers.error_user_data);
   raptor_www_set_write_bytes_handler(www, raptor_parse_uri_write_bytes, 
                                      rdf_parser);
 
@@ -847,44 +855,19 @@ raptor_parser_fatal_error(raptor_parser* parser, const char *message, ...)
   va_list arguments;
 
   va_start(arguments, message);
-
-  raptor_parser_fatal_error_varargs(parser, message, arguments);
-  
+  if(parser)
+    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_FATAL,
+                             parser->error_handlers.fatal_error_handler,
+                             parser->error_handlers.fatal_error_user_data, 
+                             &parser->locator,
+                             message, arguments); 
+  else
+    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_FATAL, NULL, NULL, NULL, 
+                             message, arguments); 
   va_end(arguments);
 }
 
 
-/*
- * raptor_parser_fatal_error_varargs - Fatal Error from a parser - Internal
- */
-void
-raptor_parser_fatal_error_varargs(raptor_parser* parser, const char *message,
-                                  va_list arguments)
-{
-  parser->failed=1;
-
-  if(parser->fatal_error_handler) {
-    char *buffer=raptor_vsnprintf(message, arguments);
-    if(!buffer) {
-      fprintf(stderr, "raptor_parser_fatal_error_varargs: Out of memory\n");
-      return;
-    }
-
-    parser->fatal_error_handler(parser->fatal_error_user_data, 
-                                &parser->locator, buffer); 
-    RAPTOR_FREE(cstring, buffer);
-    abort();
-  }
-
-  raptor_print_locator(stderr, &parser->locator);
-  fprintf(stderr, " raptor fatal error - ");
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
-
-  abort();
-}
-
-
 /*
  * raptor_parser_fatal_error_message_handler - Error from a parser - Internal
  */
@@ -897,17 +880,14 @@ raptor_parser_fatal_error_message_handler(void *user_data,
 
   parser->failed=1;
 
-  if(parser && parser->error_handler) {
-    parser->fatal_error_handler(parser->fatal_error_user_data, locator, message);
-    abort();
-  }
-
-  raptor_print_locator(stderr, locator);
-  fprintf(stderr, " raptor fatal error - ");
-  fputs(message, stderr);
-  fputc('\n', stderr);
-
-  abort();
+  if(parser)
+    raptor_log_error(RAPTOR_LOG_LEVEL_FATAL,
+                     parser->error_handlers.fatal_error_handler,
+                     parser->error_handlers.fatal_error_user_data, 
+                     &parser->locator,
+                     message); 
+  else
+    raptor_log_error(RAPTOR_LOG_LEVEL_FATAL, NULL, NULL, NULL, message); 
 }
 
 
@@ -942,11 +922,15 @@ raptor_parser_simple_error(void* user_data, const char *message, ...)
   va_start(arguments, message);
 
   if(parser)
-    raptor_invoke_message_varargs("error",
-                                  parser->error_handler,
-                                  parser->error_user_data,
-                                  &parser->locator,
-                                  message, arguments);
+    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_ERROR,
+                             parser->error_handlers.error_handler,
+                             parser->error_handlers.error_user_data,
+                             &parser->locator,
+                             message, arguments);
+  else
+    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_ERROR,
+                             NULL, NULL, NULL,
+                             message, arguments);
   
   va_end(arguments);
 }
@@ -965,11 +949,15 @@ raptor_parser_error_varargs(raptor_parser* parser, const char *message,
                             va_list arguments)
 {
   if(parser)
-    raptor_invoke_message_varargs("error",
-                                  parser->error_handler,
-                                  parser->error_user_data,
-                                  &parser->locator,
-                                  message, arguments);
+    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_ERROR,
+                             parser->error_handlers.error_handler,
+                             parser->error_handlers.error_user_data,
+                             &parser->locator,
+                             message, arguments);
+  else
+    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_ERROR,
+                             NULL, NULL, NULL,
+                             message, arguments);
 }
 
 
@@ -983,15 +971,14 @@ raptor_parser_error_message_handler(void *user_data,
 {
   raptor_parser* parser=(raptor_parser*)user_data;
 
-  if(parser && parser->error_handler) {
-    parser->error_handler(parser->error_user_data, locator, message);
-    return;
-  }
-
-  raptor_print_locator(stderr, locator);
-  fprintf(stderr, " raptor error - ");
-  fputs(message, stderr);
-  fputc('\n', stderr);
+  if(parser)
+    raptor_log_error(RAPTOR_LOG_LEVEL_ERROR,
+                     parser->error_handlers.error_handler,
+                     parser->error_handlers.error_user_data,
+                     &parser->locator,
+                     message);
+  else
+    raptor_log_error(RAPTOR_LOG_LEVEL_ERROR, NULL, NULL, NULL, message);
 }
 
 
@@ -1006,10 +993,15 @@ raptor_parser_warning(raptor_parser* parser, const char *message, ...)
   va_start(arguments, message);
 
   if(parser)
-    raptor_invoke_message_varargs("warning",
-                                  parser->warning_handler,
-                                  parser->warning_user_data, &parser->locator,
-                                  message, arguments);
+    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_WARNING,
+                             parser->error_handlers.warning_handler,
+                             parser->error_handlers.warning_user_data,
+                             &parser->locator,
+                             message, arguments);
+  else
+    raptor_log_error_varargs(RAPTOR_LOG_LEVEL_WARNING,
+                             NULL, NULL, NULL,
+                             message, arguments);
   
   va_end(arguments);
 }
@@ -1025,15 +1017,14 @@ raptor_parser_warning_message_handler(void *user_data,
 {
   raptor_parser* parser=(raptor_parser*)user_data;
 
-  if(parser && parser->warning_handler) {
-    parser->warning_handler(parser->warning_user_data, locator, message);
-    return;
-  }
-
-  raptor_print_locator(stderr, locator);
-  fprintf(stderr, " raptor warning - ");
-  fputs(message, stderr);
-  fputc('\n', stderr);
+  if(parser)
+    raptor_log_error(RAPTOR_LOG_LEVEL_WARNING,
+                     parser->error_handlers.warning_handler,
+                     parser->error_handlers.warning_user_data,
+                     &parser->locator,
+                     message);
+  else
+    raptor_log_error(RAPTOR_LOG_LEVEL_WARNING, NULL, NULL, NULL, message);
 }
 
 
@@ -1055,8 +1046,8 @@ void
 raptor_set_fatal_error_handler(raptor_parser* parser, void *user_data,
                                raptor_message_handler handler)
 {
-  parser->fatal_error_user_data=user_data;
-  parser->fatal_error_handler=handler;
+  parser->error_handlers.fatal_error_user_data=user_data;
+  parser->error_handlers.fatal_error_handler=handler;
 }
 
 
@@ -1075,8 +1066,8 @@ void
 raptor_set_error_handler(raptor_parser* parser, void *user_data,
                          raptor_message_handler handler)
 {
-  parser->error_user_data=user_data;
-  parser->error_handler=handler;
+  parser->error_handlers.error_user_data=user_data;
+  parser->error_handlers.error_handler=handler;
 }
 
 
@@ -1095,8 +1086,8 @@ void
 raptor_set_warning_handler(raptor_parser* parser, void *user_data,
                            raptor_message_handler handler)
 {
-  parser->warning_user_data=user_data;
-  parser->warning_handler=handler;
+  parser->error_handlers.warning_user_data=user_data;
+  parser->error_handlers.warning_handler=handler;
 }
 
 
@@ -1745,9 +1736,9 @@ raptor_guess_parser_name(raptor_uri *uri, const char *mime_type,
       break;
 
     if(factory->recognise_syntax)
-      score=factory->recognise_syntax(factory, buffer, len, 
-                                      identifier, suffix, 
-                                      mime_type);
+      score += factory->recognise_syntax(factory, buffer, len, 
+                                         identifier, suffix, 
+                                         mime_type);
 
     if(i > MAX_PARSERS)
       RAPTOR_FATAL2("Number of parsers greater than static buffer size %d\n",
@@ -1788,19 +1779,15 @@ raptor_parser_copy_user_state(raptor_parser *to_parser,
   int i;
   
   to_parser->user_data= from_parser->user_data;
-  to_parser->fatal_error_user_data= from_parser->fatal_error_user_data;
-  to_parser->error_user_data= from_parser->error_user_data;
-  to_parser->warning_user_data= from_parser->warning_user_data;
-  to_parser->fatal_error_handler= from_parser->fatal_error_handler;
-  to_parser->error_handler= from_parser->error_handler;
-  to_parser->warning_handler= from_parser->warning_handler;
+  memcpy(&to_parser->error_handlers, &from_parser->error_handlers,
+         sizeof(raptor_error_handlers));
   to_parser->statement_handler= from_parser->statement_handler;
   to_parser->generate_id_handler_user_data= from_parser->generate_id_handler_user_data;
   to_parser->generate_id_handler= from_parser->generate_id_handler;
   to_parser->default_generate_id_handler_base= from_parser->default_generate_id_handler_base;
   /* copy over non-shared user state - generate ID prefix string */
   if(from_parser->default_generate_id_handler_prefix) {
-    to_parser->default_generate_id_handler_prefix=RAPTOR_MALLOC(cstring,
+    to_parser->default_generate_id_handler_prefix=(char*)RAPTOR_MALLOC(cstring,
                     from_parser->default_generate_id_handler_prefix_length+1);
     strncpy((char*)to_parser->default_generate_id_handler_prefix, 
             (const char*)from_parser->default_generate_id_handler_prefix,
@@ -1975,6 +1962,39 @@ raptor_parser_get_accept_header_all(void)
 }
 
 
+void
+raptor_parser_save_content(raptor_parser* rdf_parser, int save)
+{
+  if(rdf_parser->sb)
+    raptor_free_stringbuffer(rdf_parser->sb);
+
+  rdf_parser->sb= save ? raptor_new_stringbuffer() : NULL;
+}
+
+
+const unsigned char*
+raptor_parser_get_content(raptor_parser* rdf_parser, size_t* length_p)
+{
+  unsigned char* buffer;
+  size_t len;
+  
+  if(!rdf_parser->sb)
+    return NULL;
+  
+  len=raptor_stringbuffer_length(rdf_parser->sb);
+  buffer=(unsigned char*)RAPTOR_MALLOC(cstring, len+1);
+  if(!buffer)
+    return NULL;
+
+  raptor_stringbuffer_copy_to_string(rdf_parser->sb, buffer, len);
+
+  if(length_p)
+    *length_p=len;
+
+  return buffer;
+}
+
+
 /* end not STANDALONE */
 #endif
 
