@@ -2,7 +2,7 @@
  *
  * raptor_rss.c - Raptor RSS tag soup parser
  *
- * Copyright (C) 2003-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2003-2007, David Beckett http://purl.org/net/dajobe/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * Contributions:
@@ -92,6 +92,9 @@ struct raptor_rss_parser_s {
 
   /* non-0 if this is an atom 1.0 parser */
   int is_atom;
+
+  /* namespaces declared here */
+  raptor_namespace* nspaces[RAPTOR_RSS_NAMESPACES_SIZE];
 };
 
 typedef struct raptor_rss_parser_s raptor_rss_parser;
@@ -186,13 +189,10 @@ raptor_rss_parse_init(raptor_parser* rdf_parser, const char *name)
     if(prefix && uri)
       nspace=raptor_new_namespace_from_uri(rss_parser->nstack,
                                            prefix, uri, 0);
-    raptor_rss_namespaces_info[n].nspace=nspace;
+    rss_parser->nspaces[n]=nspace;
   }
 
-  sax2=raptor_new_sax2(rdf_parser, 
-                       rdf_parser, raptor_parser_error_message_handler,
-                       rdf_parser, raptor_parser_fatal_error_message_handler,
-                       rdf_parser, raptor_parser_warning_message_handler);
+  sax2=raptor_new_sax2(rdf_parser, &rdf_parser->error_handlers);
   rss_parser->sax2=sax2;
 
   raptor_sax2_set_start_element_handler(sax2, raptor_rss_start_element_handler);
@@ -201,8 +201,6 @@ raptor_rss_parse_init(raptor_parser* rdf_parser, const char *name)
   raptor_sax2_set_cdata_handler(sax2, raptor_rss_cdata_handler);
   raptor_sax2_set_comment_handler(sax2, raptor_rss_comment_handler);
 
-  raptor_sax2_set_locator(sax2, &rdf_parser->locator);
-
   return 0;
 }
 
@@ -220,8 +218,8 @@ raptor_rss_parse_terminate(raptor_parser *rdf_parser)
 
   /* Initialise the namespaces */
   for(n=0; n < RAPTOR_RSS_NAMESPACES_SIZE; n++) {
-    if(raptor_rss_namespaces_info[n].nspace)
-      raptor_free_namespace(raptor_rss_namespaces_info[n].nspace);
+    if(rss_parser->nspaces[n])
+      raptor_free_namespace(rss_parser->nspaces[n]);
   }
 
   if(rss_parser->nstack)
@@ -1274,12 +1272,14 @@ raptor_rss_parse_recognise_syntax(raptor_parser_factory* factory,
   }
   
   if(mime_type) {
-    if(strstr((const char*)mime_type, "rss"))
-      score+=4;
-    else if(strstr((const char*)mime_type, "xml"))
-      score+=4;
-    else if(strstr((const char*)mime_type, "atom"))
-      score+=4;
+    if(!strstr((const char*)mime_type, "html")) {
+      if(strstr((const char*)mime_type, "rss"))
+        score+=4;
+      else if(strstr((const char*)mime_type, "xml"))
+        score+=4;
+      else if(strstr((const char*)mime_type, "atom"))
+        score+=4;
+    }
   }
   
   return score;
